/***********************************************************************
 * File: unit_test.c
 * 	(Extracted from
 *	   "$Id: unit_test.c,v 1.5 2006/04/13 22:02:10 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source code implements the unit test for sql_quoting().
 *
 * Copyright (C) 2006 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

/* include system definitions */
#include <stdlib.h>
#include <stdio.h>
extern int snprintf(char *str, size_t size, const char *format, ...);

/* include definitions for sql_quoting() API */
#include "sql_quoting.h"

int main(int ac, char **av)
{
	char			*enclosed[] = { "", "'", "\"" }
				, escapes[] = { '\0', '\\', '\t', '@' }
				, val[15]
				, src[sizeof(val)]
				, dst[(sizeof(src) - 1) * 2 + 3]
				, *result, *errmsg
				;
	SQL_quoting_qualifier_t	w, e, pat, enc, sta, vert
				, qualifier = SQL_quoting_default
				;
	size_t			en, qn, dst_sz, escn;
	int			sv_errno;

	printf("{{{ test for sql_quoting() starts...\n\n"
		); fflush(stdout);

	strcpy(val, "100%_|ove\\" SQL_quoting_CONTROL_CHARACTERS);

	for (en = 0; en < sizeof(enclosed) / sizeof(enclosed[0]); ++en)
	for (qn = 0; qn < sizeof(enclosed) / sizeof(enclosed[0]); ++qn)
	for (vert = 0x0
	; vert <= SQL_quoting_escape_vertical_bar
	; vert += SQL_quoting_escape_vertical_bar)
	for (sta = 0x0
	; sta <= SQL_quoting_do_not_escape_quots
	; sta += SQL_quoting_do_not_escape_quots)
	for (enc = 0x0
	; enc <= SQL_quoting_enclose_result
	; enc += SQL_quoting_enclose_result)
	for (pat = 0x0
	; pat <= SQL_quoting_escape_pattern_codes
	; pat += SQL_quoting_escape_pattern_codes)
	for (e = 0x0
	; e <= SQL_quoting_escape_control_codes
	; e += SQL_quoting_escape_control_codes)
	for (w = 0x0
	; w <= SQL_quoting_with_backslash
	; w += SQL_quoting_with_backslash)
	for (dst_sz = sizeof(dst)
	; (dst_sz >= sizeof(src)) && dst_sz <= sizeof(dst)
	; dst_sz -= sizeof(src))
	for (escn = 0
	; escn < sizeof(escapes) / sizeof(escapes[0])
	; ++escn) {
		errno	= 0;
		snprintf(src, sizeof(src), "%s%s%s"
			, enclosed[en], val, enclosed[en]
			);
		qualifier	= (w | e | pat | enc | sta);
		memset(dst, '@', sizeof(dst) - 1);
		dst[sizeof(dst) - 1]	= '\0';
		result	= sql_quoting(src
					, qualifier
					, *(enclosed[qn])
					, escapes[escn]
					, dst
					, dst_sz
					);
		sv_errno	= errno;
		errmsg	= strerror(sv_errno);
		printf("sql_quoting(src@%p [%s] (len = %d), "
			"qualifier = 0x%x, "
			"quote_char = '%c', escape_code = 0x%x, o"
			"dst@%p, "
			"dst_sz = %d)\n"
			"...returns %p [%s] (len = %d)"
			"(errno = %d [%s]).\n\n"
			,	src, src, strlen(src)
			,	qualifier
			,	*(enclosed[qn]),	escapes[escn]
			,	dst
			,	dst_sz
			,	result, ((result == NULL)? "": result)
			,				strlen(dst)
			,	sv_errno, errmsg
			); fflush(stdout);
	}

	printf("}}} test for sql_quoting() finished!\n"
		); fflush(stdout);

	exit(0); return 0;
}
