/***********************************************************************
 * File: fair_copy.h
 * 	(Extracted from
 *	   "$Id: fair_copy.h,v 1.4 2006/05/03 15:37:25 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header defines the `fair_copy()" API prototyping.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _FAIR_COPY_H_ /* { */
#define _FAIR_COPY_H_

#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>

/*
 * This API processes the source string to fit the rows X cols box area.
 *
 * arguments:
 *	src is pointing to the source string.
 *	linectl indicates that text-rectangle needs the new-line codes
 *	for the each end-of-line (Fair_copy_line_breaking) or not
 *	(Fair_copy_wrap_around).
 *	rows indicates the number of the rows of the box area.
 *	cols indicates the number of columns of the box area.
 *	dst_sz indicates the size of the destiny string.
 *	dst is pointing to the destiny (outputs processed string).
 *	processed_rows outputs the rows of dst string.
 *
 * returns:
 *	Zero:	Normal end
 *	1:	Illegal argument
 *	2:	Failed to allocate the duplicated source string
 *	3:	Rows of output is greater than 'rows'
 *	4:	Output is overflow the destiny string size.
 */
typedef enum fair_copy_line_control {
	  Fair_copy_line_breaking
	, Fair_copy_wrap_around
	} Fair_copy_line_control;

extern int fair_copy(char *src
		, Fair_copy_line_control linectl
		, size_t rows
		, size_t cols
		, size_t dst_sz
		, char *dst
		, size_t *processed_rows
		);
#	define FAIR_COPY_NORMAL_END		0
#	define FAIR_COPY_ILLEGAL_ARGS		1
#	define FAIR_COPY_NO_MEMORY		2
#	define FAIR_COPY_ROWS_OVER		3

#endif /* } !defined(_FAIR_COPY_H_) */
